package com.example.venue_reservation_service.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.example.venue_reservation_service.domain.*;
import com.example.venue_reservation_service.dto.RAddDTO;
import com.example.venue_reservation_service.dto.ReservationDTO;
import com.example.venue_reservation_service.mapper.*;
import com.example.venue_reservation_service.service.*;
import com.example.venue_reservation_service.utils.DateUtil;
import com.example.venue_reservation_service.utils.DownExcel;
import com.example.venue_reservation_service.utils.ScheduleUtil;
import com.example.venue_reservation_service.vo.*;
import com.example.venue_reservation_service.vo.excel.ReservationExcel;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletResponse;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.IOException;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.temporal.TemporalAdjusters;
import java.util.*;
import java.util.stream.Collectors;

/**
* @author 31586
* @description 针对表【venue_reservation】的数据库操作Service实现
* @createDate 2024-11-13 11:01:43
*/
@Service
public class ReservationServiceImpl extends ServiceImpl<ReservationMapper, Reservation>
    implements ReservationService{

    @Resource
    private BalanceService balanceService;

    @Resource
    private PaymentService paymentService;

    @Resource
    private InformationMapper informationMapper;


    @Resource
    private TypeMapper typeMapper;

    @Resource
    private AdminMapper adminMapper;

    @Resource
    private InformationService informationService;

    @Resource
    private UserService userService;

    @Resource
    private SchoolMapper schoolMapper;

    @Resource
    private ScheduleUtil scheduleUtil ;

    @Resource
    private SlotMapper slotMapper;

    @Resource
    private VenueSlotService venueSlotService;



    @Transactional
    @Override
    public Result addReservation(RAddDTO dto) {

        // 校验日期是否合适
        LocalDate reservationDate = dto.getReservationDate();
        if(reservationDate.isBefore(LocalDate.now())){
            // 如果预约日期早于当前日期
            return Result.fail().message("预约日期需在"+LocalDate.now()+"及之后");
        }

        // 判断场地是否存在
        Information information = informationService.getById(dto.getVenueId());

        if (Optional.ofNullable(information).isEmpty()) {
            return Result.fail().message("场地数据不存在");
        }

        // 判断用户是否有待签到的场地预约信息
        if (Optional.ofNullable(getOne(Wrappers.<Reservation>lambdaQuery()
                        .eq(Reservation::getUserId, dto.getUserId())
                        .eq(Reservation::getAuditStatus, 1)
                        .eq(Reservation::getReservationDate, reservationDate)))
                .isPresent()) {
            return Result.fail().message( reservationDate +"存在待签到的订单");
        }

        // 获取此次预约的时间段
        List<TimeItem> times = dto.getTimes();
        if (times.size() == 0) {
            return Result.fail().message("预约时间段不能为空");
        }

        // 使用 Stream API 进行排序
        List<TimeItem> sortedTimeItems = times.stream()
                .sorted(Comparator.comparingInt(TimeItem::getId)).toList();
        // 取出预约时间的开始和结束时间
        LocalTime start = sortedTimeItems.get(0).getStartTime();
        LocalTime end = sortedTimeItems.get(sortedTimeItems.size() - 1).getEndTime();

        // 如果是今天进行预约
        if(reservationDate.compareTo(LocalDate.now()) == 0){
            // 校验事件
            if (LocalTime.now().isAfter(end)){
                return Result.fail().message("无效预约时间");
            }
        }

        // 检查当前时间段内场地是否被禁用
        List<Slot> slots = slotMapper.selectList(Wrappers.<Slot>lambdaQuery()
                .eq(Slot::getReservationDate, reservationDate)
                .and(wrap -> {  // 嵌套 OR 条件：( (slot_start < ? AND slot_end > ? ) OR ... )
                    wrap.or(sub -> sub
                                    .lt(Slot::getSlotStart, start)
                                    .gt(Slot::getSlotEnd, start)
                            )
                            .or(sub -> sub
                                    .lt(Slot::getSlotStart, end)
                                    .gt(Slot::getSlotEnd, end)
                            )
                            .or(sub -> sub
                                    .le(Slot::getSlotStart, start)
                                    .ge(Slot::getSlotEnd, end)
                            );
                }));
        for (Slot slot : slots) {
            List<VenueSlot> list = venueSlotService.list(Wrappers.<VenueSlot>lambdaQuery()
                    .eq(VenueSlot::getVenueId, information.getId())
                    .eq(VenueSlot::getSlotId, slot.getId()));
            if(list.size() > 0){
                // 存在禁用时间段，不可预约
                return Result.fail().message("预约时间段内存在禁用时间段");
            }
        }

        // 判断该预约时间是否合理（即查询用户预约的日期下，该场地的此时间段内是否存在其他的预约订单）
        int size = list(Wrappers.<Reservation>lambdaQuery()
                .eq(Reservation::getReservationDate, reservationDate)
                .eq(Reservation::getVenueId, dto.getVenueId())
                .in(Reservation::getAuditStatus,1, 2 )
                .and(wrap -> {  // 嵌套 OR 条件：( (slot_start < ? AND slot_end > ? ) OR ... )
                    wrap.or(sub -> sub
                                    .lt(Reservation::getStartTime, start)
                                    .gt(Reservation::getEndTime, start)
                            )
                            .or(sub -> sub
                                    .lt(Reservation::getStartTime, end)
                                    .gt(Reservation::getEndTime, end)
                            )
                            .or(sub -> sub
                                    .le(Reservation::getStartTime, start)
                                    .ge(Reservation::getEndTime, end)
                            );
                }))
                .size();
        if (size>0) {
            // 存在其他订单数据，不可预约
            return Result.fail().message("预约时间段内存在其他用户订单");
        }

        // 检查用户账号数据
        // 获取用户账号余额
        Balance balance = balanceService.getOne(Wrappers.<Balance>lambdaQuery()
                .eq(Balance::getUserId, dto.getUserId()));
        Double aDouble = balance.getBalance();

        // 获取场地的使用费用
        Double money = dto.getMoney();
        // 判断用户余额是否可以完成此次支付
        if(aDouble < money){
            return Result.fail().message("账户余额不足，请进行充值");
        }

        // 修改账户余额
        aDouble -= money;
        balance.setBalance(aDouble);
        // 生成支付记录
        Payment payment = new Payment(dto.getUserId(), money, "支出", LocalDateTime.now());
        paymentService.save(payment);

        balanceService.updateById(balance);

        // 添加预约信息
        Reservation reservation = BeanUtil.copyProperties(dto, Reservation.class);
        // 将预约状态设置为 1 即待签到
        reservation.setAuditStatus(1);
        // 设置当前预约对应的支付信息
        reservation.setPayId(payment.getId());
        // 设置预约开始和结束时间
        reservation.setStartTime(start);
        reservation.setEndTime(end);
        // 保存预约信息
        save(reservation);

        // 设置定时任务（即到预约结束时间进行检查签到情况）
        // 任务
        Runnable task = () -> {
            // 处理当前预约
            dealReservation(reservation.getId());
        };
        scheduleUtil.scheduleOneTimeTask(task, reservationDate, end);

        // 返回订单信息
        return Result.ok(payment).message("预约成功");
    }

    @Override
    @Async
    public void dealReservation(Integer reservationId) {
        Reservation reservation = getById(reservationId);
        // 判空操作
        if (Optional.ofNullable(reservation).isPresent()) {
            Integer status = reservation.getAuditStatus();
            // 未签到：将该条预约数据的状态设置为 -2，同时归还支付金额
            if(status == 1){
                reservation.setAuditStatus(-2);
                updateById(reservation);
                // 修改交易记录
                Payment payment = paymentService.getById(reservation.getPayId());
                payment.setSituation("过期");
                paymentService.updateById(payment);
                // 归还支付金额
                balanceService.returnMoney(reservation.getUserId(), payment.getAmount());
            }

        }
    }

    @Override
    public Result getReservations(ReservationDTO dto) {
        Page<Reservation> page = new Page<>(dto.getCurrent(), dto.getSize());
        LambdaQueryWrapper<Reservation> wrapper = new LambdaQueryWrapper<>();

        Integer status = dto.getStatus();
        if(status < -2 || status > 5){
            return Result.fail().message("非法预约状态");
        }

        System.out.println(dto);
        if (status != 0) {
            wrapper.eq(Reservation::getAuditStatus, status);
        }

        Integer userId = dto.getUserId();
        Integer userType = userService.getById(userId).getType();
        if (userType != 0) {
            // 判断是否为管理员
            if(userType == 3) {
                // 为管理员，获取管理的场馆信息
                Integer typeId = adminMapper.selectOne(Wrappers.<Admin>lambdaQuery().eq(Admin::getUserId, userId)).getTypeId();
                // 获取场馆下场地列表
                List<Integer> venueIds = informationService.getIdsByTypeId(typeId);
                wrapper.in(Reservation::getVenueId, venueIds);
            } else {
                // 用户
                wrapper.eq(Reservation::getUserId, userId);
            }
        }else {
            wrapper.eq(Reservation::getUserId, userId);
        }

        wrapper.orderByDesc(Reservation::getReservationDate);
        page = page(page, wrapper);


        List<ReservationVO> list = new ArrayList<>();
        for (Reservation reservation : page.getRecords()) {
            // 获取场地名称
            Information information = informationMapper.selectOne(Wrappers
                    .<Information>lambdaQuery()
                    .eq(Information::getId, reservation.getVenueId()));

            // 获取对应场馆名称
            Type type = typeMapper.selectOne(Wrappers
                    .<Type>lambdaQuery()
                    .eq(Type::getId, information.getTypeId()));
            // 预约时间段
            String timeStr = reservation.getStartTime() + "~" + reservation.getEndTime();

            // 获取支付信息
            Payment payment = paymentService.getById(reservation.getPayId());

            ReservationVO result = new ReservationVO(
                    reservation.getId(),
                    information.getId(),
                    information.getType(),
                    information.getVenueName(),
                    timeStr,
                    reservation.getReservationDate() + "",
                    type.getVname(),
                    payment.getId(),
                    DateUtil.getEpochMilli(payment.getTime())+ "-" + payment.getId(),
                    payment.getAmount(),
                    payment.getTime()
            );
            list.add(result);
        }
        QueryVO<ReservationVO> vo = new QueryVO<>(list, page.getTotal());
        return Result.ok(vo).message("查询成功");
    }

    @Override
    public Result cancelReservation(Integer reservationId) {
        Reservation reservation = getById(reservationId);

        if (Optional.ofNullable(reservation).isEmpty()) {
            return Result.fail().message("无订单信息");
        }

        if (reservation.getAuditStatus() !=1) {
            return Result.fail().message("仅当订单待签到才可以进行取消");
        }

        if (reservation.getReservationDate().compareTo(LocalDate.now()) == 0) {
            // 获取订单日期
            if (LocalTime.now().isAfter(reservation.getStartTime()) && LocalTime.now().isBefore(reservation.getStartTime())) {
                // 对于当天使用中的订单
                // 如果是未签到，可以进行取消（后期如果引入信用机制扣除荣誉分）
                Integer status = reservation.getAuditStatus();
                if (status == 2) {
                    return Result.fail().message("订单已签到");
                }
            }
        }

        // 将预约订单状态设置为 -1，即表示取消订单
        reservation.setAuditStatus(-1);
        updateById(reservation);
        // 归还支付金额
        Payment payment = paymentService.getById(reservation.getPayId());
        // 修改支付情况
        payment.setSituation("取消");
        paymentService.updateById(payment);
        // 修改用户账户余额
        balanceService.returnMoney(reservation.getUserId(), payment.getAmount());

        return Result.ok().message("预约订单取消成功");
    }

    @Override
    public Result signInReservation(Integer reservationId) {
        Reservation reservation = getById(reservationId);

        if (Optional.ofNullable(reservation).isEmpty()) {
            return Result.fail().message("预约信息不存在");
        }

        if (reservation.getAuditStatus() != 1) {
            return Result.fail().message("预约信息不为待签到状态");
        }

        // 签到时间可以在预约开始时间提前 15 分钟 或者 在预约时间段内
        // 获取预约开始时间段
        LocalTime startTime = reservation.getStartTime();

        if (reservation.getReservationDate().compareTo(LocalDate.now()) != 0) {
            return Result.fail().message("非当天订单");
        }
        // 判断当前时间是否合理
        if (reservation.getEndTime().isBefore(LocalTime.now())){
            return Result.fail().message("订单已经过期");
        }

        // 提前签到
        if (startTime.minusMinutes(15).isAfter(LocalTime.now())) {
            return Result.fail().message("无效签到时间");
        }

        // 正常时间段签到
        // 将当前签到订单设置
        reservation.setAuditStatus(2);
        reservation.setCheckInTime(LocalDateTime.now());
        updateById(reservation);

        // 返回预约信息
        return Result.ok(reservation).message("订单签到成功");
    }

    @Override
    public Result queryDetail(Integer id) {
        Reservation reservation = getById(id);
        if (Optional.ofNullable(reservation).isEmpty()) {
            return Result.fail().message("未查询到预约信息");
        }

        // 加载场地信息
        Information information = informationService.getById(reservation.getVenueId());

        // 加载支付信息
        Payment payment = paymentService.getById(reservation.getPayId());

        ReservationDetail detail = new ReservationDetail(
                information.getVenueName(),
                DateUtil.getEpochMilli(payment.getTime()) + id,
                reservation.getReservationDate(),
                reservation.getStartTime() + "~" + reservation.getEndTime(),
                payment.getAmount(),
                payment.getTime()
        );

        return Result.ok(detail).message("ok");
    }

    @Override
    public Result queryData(String college) {

        List<String> xlist = new ArrayList<>();
        List<Long> ylist = new ArrayList<>();

        // 查询每个单位的用户编号集合
        QueryWrapper<School> wrapper = new QueryWrapper<>();
        wrapper.select("distinct user_id")
                .eq("college", college);
        List<Integer> userIds = schoolMapper.selectObjs(wrapper)
                .stream()
                .map(obj -> (Integer) obj)
                .toList();
        // 查询当前单位下预约次数（最近3个月）
        for (int i = 2;i>=0;i--){
            LocalDate date = LocalDate.now().minusMonths(i);
            Long count = getBaseMapper().selectCount(Wrappers.<Reservation>lambdaQuery()
                    .in(Reservation::getUserId, userIds)
                    .between(Reservation::getReservationDate,
                            date.with(TemporalAdjusters.firstDayOfMonth()),
                            date.with(TemporalAdjusters.lastDayOfMonth())));
            xlist.add(date.getYear()+"年" + date.getMonthValue() +"月");
            ylist.add(count);
        }


        return Result.ok(new DataVO(xlist, ylist)).message("ok");
    }

    @Override
    public void exportExcel(HttpServletResponse response, Integer userId) {
        ReservationMapper mapper = getBaseMapper();
        try {
            DownExcel.download(response, ReservationExcel.class, mapper.selectByUserId(userId));
        } catch (IOException | IllegalAccessException | InstantiationException e) {
            throw new RuntimeException(e);
        }
    }


}




